import 'package:agschat/src/constants.dart';
import 'package:agschat/src/core/services/stream_socket.dart';
import 'package:agschat/src/helper/custom_styles.dart';
import 'package:agschat/src/helper/images.dart';
import 'package:agschat/src/helper/utilites.dart';
import 'package:agschat/src/view/contact_list_view.dart';
import 'package:agschat/src/view/message_view.dart';
import 'package:agschat/src/view/profile_view.dart';
import 'package:agschat/src/view/widgets/comman.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';

class ChatsListView extends StatefulWidget {
  const ChatsListView({Key? key}) : super(key: key);

  _ChatsListViewState createState() => _ChatsListViewState();
}

class _ChatsListViewState extends State<ChatsListView> {
  TextEditingController searchController = TextEditingController();
  List snap = [];
  String searchText = '';

  @override
  void initState() {
    super.initState();
    getProfileDetail(
      (val) {
        print(val);
        myGroups.value = val['groups'] ?? [];
        myMobileEmail.value = val['mobile_email'];
        myUserName.value = val['name'];
        myProfilePicture.value = val['profilePicture'];
      },
    );
    getChatsList();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Column(
          children: [
            appBar(),
            Padding(
                padding: const EdgeInsets.only(left: 20, right: 20, top: 8),
                child: TextField(
                  controller: searchController,
                  onChanged: (val) {
                    setState(() {
                      searchText = val;
                    });
                  },
                  decoration: InputDecoration(
                    hintText: "Search chat",
                    hintStyle: CustomTextStyles.medium(
                        fontSize: 14.0, fontColor: Colors.black38),
                    filled: true,
                    fillColor: const Color(0xffEEEEEF),
                    border: searchCommonBorderStyle,
                    enabledBorder: searchCommonBorderStyle,
                    disabledBorder: searchCommonBorderStyle,
                    focusedBorder: searchCommonBorderStyle,
                    focusedErrorBorder: searchCommonBorderStyle,
                    errorBorder: searchCommonBorderStyle,
                    contentPadding: const EdgeInsets.only(left: 6, right: 6),
                    prefixIcon: const Padding(
                        padding: EdgeInsetsDirectional.only(start: 6.0),
                        child: Icon(Icons.search)),
                  ),
                )),
            const Padding(
              padding: EdgeInsets.only(
                top: 4,
              ),
              child: Divider(
                color: Color(0xffd4d4d4),
                thickness: 1,
              ),
            ),
            Expanded(
              child: StreamBuilder(
                  stream: chatListSoket.getResponse,
                  builder:
                      (BuildContext context, AsyncSnapshot<dynamic> snapshot) {
                    if (snapshot.connectionState == ConnectionState.waiting) {
                      return const Center(child: CircularProgressIndicator());
                    }
                    snap = snapshot.data as List;
                    if (searchText.isNotEmpty) {
                      snap = snap.where((element) {
                        return element['name']
                            .toString()
                            .toLowerCase()
                            .contains(searchText.toLowerCase());
                      }).toList();
                    }
                    return snap.isEmpty
                        ? const Center(
                            child: Text("Data not found"),
                          )
                        : ListView.separated(
                            padding: const EdgeInsets.symmetric(vertical: 10),
                            itemBuilder: (context, index) {
                              return chatItem(snap[index]);
                            },
                            separatorBuilder: (context, index) {
                              return const Padding(
                                padding: EdgeInsets.only(top: 10, bottom: 10),
                                child: Divider(
                                  color: Color(0xffd4d4d4),
                                  thickness: 1.0,
                                  height: 1.0,
                                ),
                              );
                            },
                            itemCount: snap.length);
                  }),
            ),
          ],
        ),
      ),
    );
  }

  Widget appBar() {
    return Container(
      color: const Color(0xffE5E5EA),
      padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 10),
      child: Row(
        children: [
          GestureDetector(
            onTap: () {
              // Get.to(() => const ProfileView());
              Navigator.push(
                context,
                MaterialPageRoute(builder: (context) => const ProfileView()),
              );
            },
            child: Common.profileNetworkImage(
                borderRadius: 60.0, height: 40.0, width: 40.0, imageUrl: ""),
          ),
          const Expanded(
            child: Text(""),
          ),
          IconButton(
            icon: SvgPicture.asset(ImageResource.oneToChatIc),
            onPressed: () {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => ContactListView(forMakeGroup: true)),
              );
            },
          ),
          IconButton(
            icon:
                SvgPicture.asset('assets/create_group.svg', package: 'agschat'),
            onPressed: () {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => ContactListView(forMakeGroup: false)),
              );
            },
          ),
          Image.asset('assets/image_placeholder.png',
              package: 'ags_chat_plugin', height: 28, width: 28)
        ],
      ),
    );
  }

  // Common border for textField
  var searchCommonBorderStyle = OutlineInputBorder(
      borderSide: const BorderSide(
        color: Color(0xffe6e6e6),
      ),
      borderRadius: BorderRadius.circular(16));

  chatItem(data) {
    var count = 0;
    if (data != null) {
      data['readCount'].forEach((va) {
        if (va['userId'] == myUserId) {
          count = va['unreadCount'];
        }
      });
    }
    return Padding(
      padding: const EdgeInsets.only(left: 12, right: 12),
      child: InkWell(
        onTap: () {
          // Get.to(() => const MessageView(), arguments: [
          //   myUserId,
          //   data,
          // ]);
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => MessageView(
                      userId: myUserId,
                      data: data,
                    )),
          );
        },
        child: Row(
          children: [
            Common.profileNetworkImage(
                borderRadius: 56.0,
                height: 56.0,
                width: 56.0,
                placeHolder: data['isGroup']
                    ? ImageResource.groupPlaceHolder
                    : 'assets/image_placeholder.png',
                imageUrl: data['groupImage']),
            const SizedBox(width: 10),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    children: [
                      Expanded(
                        child: Text(
                          data['name'] == "" ? "-" : data['name'],
                          style: CustomTextStyles.semiBold(
                            fontSize: 18.0,
                            fontColor: Colors.black,
                          ),
                        ),
                      ),
                      const SizedBox(width: 10),
                      data['recentMessage']['timeMilliSeconds'] == null
                          ? const SizedBox.shrink()
                          : Text(
                              Utilities.convertTimeStampToDate(
                                  data['recentMessage']['timeMilliSeconds']
                                          ['seconds'] ??
                                      ""),
                              style: CustomTextStyles.normal(
                                  fontSize: 10.0, fontColor: Colors.black),
                            )
                    ],
                  ),
                  const SizedBox(height: 8),
                  data['isGroup']
                      ? Text(
                          "Start your conversation",
                          style: CustomTextStyles.normal(
                              fontSize: 12.0, fontColor: Colors.grey),
                        )
                      : Row(
                          children: [
                            data['recentMessage']['type'] == "text" ||
                                    data['recentMessage']['type'] == null
                                ? const SizedBox.shrink()
                                : Image.asset(
                                    data['recentMessage']['type'] == "audio"
                                        ? ImageResource.audioIc
                                        : data['recentMessage']['type'] ==
                                                "video"
                                            ? ImageResource.videoIc
                                            : data['recentMessage'] == "image"
                                                ? ImageResource.imageIc
                                                : ImageResource.documentIc,
                                    height: 20,
                                    width: 20,
                                  ),
                            Expanded(
                              child: Text(
                                data['recentMessage']['message'] == null
                                    ? "Start your conversation"
                                    : data['recentMessage']['type'] == "text"
                                        ? data['recentMessage']['message']
                                        : " ${data['recentMessage']['type']}",
                                style: CustomTextStyles.normal(
                                  fontSize: 12.0,
                                  fontColor: Colors.grey,
                                ),
                              ),
                            ),
                            count == 0
                                ? const SizedBox.shrink()
                                : CircleAvatar(
                                    backgroundColor: Colors.red,
                                    radius: 10,
                                    child: Text(
                                      "$count",
                                      style: CustomTextStyles.medium(
                                        fontSize: 10.0,
                                        fontColor: Colors.white,
                                      ),
                                    ),
                                  )
                          ],
                        )
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}
