import 'dart:io';

import 'package:agschat/src/constants.dart';
import 'package:agschat/src/core/services/stream_socket.dart';
import 'package:agschat/src/helper/colors.dart';
import 'package:agschat/src/helper/images.dart';
import 'package:agschat/src/view/contact_list_view.dart';
import 'package:agschat/src/view/widgets/comman.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../../helper/custom_styles.dart';

class GroupInfoView extends StatefulWidget {
  GroupInfoView({Key? key, this.information}) : super(key: key);
  var information;
  @override
  State<GroupInfoView> createState() => _GroupInfoViewState();
}

class _GroupInfoViewState extends State<GroupInfoView> {
  var groupDetails;
  var groupName = "".obs;
  var participentCounts = "".obs;
  var participentList = [].obs;
  var createdBy = "".obs;
  var groupImage = "".obs;
  var groupId = "".obs;
  var isEditable = false.obs;
  TextEditingController groupNameController = TextEditingController();
  var loadData = true.obs;
  @override
  void initState() {
    if (widget.information != null) {
      groupDetails = widget.information;
      print(groupDetails);
      print(groupDetails['profilePicture']);

      groupId.value = groupDetails['groupId'];
    }
    getGroupDetail(groupId.value, (data) {
      createdBy.value = data['createdBy'];
      groupName.value = data['name'];
      groupNameController.text = groupName.value;
      participentList.value = data['users'];
      groupImage.value = data['groupImage'] ?? "";
      loadData.value = false;
    });
    super.initState();
  }

  var imageBytes;
  var profileImage = File("").obs;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      resizeToAvoidBottomInset: false,
      body: SafeArea(
        child: Container(
          color: const Color(0xffE5E5EA),
          child: Obx(
            () => loadData.value
                ? Center(
                    child: const CircularProgressIndicator(),
                  )
                : SingleChildScrollView(
                    child: Column(
                      children: [
                        appBar(),
                        Container(
                          width: Get.width,
                          color: Colors.white,
                          child: Column(
                            children: [
                              const SizedBox(
                                height: 24,
                              ),
                              InkWell(
                                onTap: isEditable.value
                                    ? () async {
                                        var pickedFile;
                                        FilePickerResult? result =
                                            await FilePicker.platform.pickFiles(
                                          type: FileType.media,
                                          withData: true,
                                        );
                                        imageBytes = result!.files[0].bytes;
                                        pickedFile =
                                            File(result.paths[0].toString());
                                        profileImage.value = pickedFile;
                                      }
                                    : null,
                                child: Obx(
                                  () => Stack(
                                    alignment: AlignmentDirectional.center,
                                    children: [
                                      !profileImage.value.isAbsolute
                                          ? Common.profileNetworkImage(
                                              borderRadius: 200.0,
                                              height: 200.0,
                                              width: 200.0,
                                              placeHolder: ImageResource
                                                  .groupPlaceHolder,
                                              imageUrl: groupImage.value,
                                            )
                                          : ClipRRect(
                                              borderRadius:
                                                  BorderRadius.circular(200),
                                              child: Container(
                                                color: Colors.black12,
                                                child: Image.file(
                                                  File(
                                                    profileImage.value.path,
                                                  ),
                                                  height: 200.0,
                                                  width: 200.0,
                                                ),
                                              ),
                                            ),
                                      isEditable.value
                                          ? const Icon(
                                              Icons.camera_alt_outlined,
                                              color: Colors.white,
                                              size: 30.0,
                                            )
                                          : SizedBox.shrink()
                                    ],
                                  ),
                                ),
                              ),
                              const SizedBox(
                                height: 16,
                              ),
                              Obx(
                                () => isEditable.value
                                    ? Padding(
                                        padding: const EdgeInsets.only(
                                            left: 16.0, right: 16.0),
                                        child: TextField(
                                          controller: groupNameController,
                                          decoration: InputDecoration(
                                            hintText: "Enter User Name",
                                            hintStyle: CustomTextStyles.medium(
                                                fontSize: 14.0,
                                                fontColor: Colors.black38),
                                            filled: true,
                                            fillColor: const Color(0xffEEEEEF),
                                            border: searchCommonBorderStyle,
                                            enabledBorder:
                                                searchCommonBorderStyle,
                                            disabledBorder:
                                                searchCommonBorderStyle,
                                            focusedBorder:
                                                searchCommonBorderStyle,
                                            focusedErrorBorder:
                                                searchCommonBorderStyle,
                                            errorBorder:
                                                searchCommonBorderStyle,
                                            contentPadding:
                                                const EdgeInsets.only(
                                                    left: 6, right: 6),
                                          ),
                                        ),
                                      )
                                    : Text(
                                        groupName.value,
                                        style: CustomTextStyles.medium(
                                            fontSize: 26.0,
                                            fontColor: Colors.black),
                                      ),
                              ),
                              const Padding(
                                padding:
                                    EdgeInsets.only(top: 10.0, bottom: 10.0),
                                child:
                                    Divider(color: Colors.black26, height: 1.0),
                              ),
                              Text(
                                "Group - ${participentList.length} participants",
                                style: CustomTextStyles.normal(
                                    fontSize: 20.0, fontColor: Colors.black),
                              ),
                              const SizedBox(
                                height: 16,
                              ),
                            ],
                          ),
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        Container(
                          color: Colors.white,
                          child: Column(
                            children: [
                              Padding(
                                padding: const EdgeInsets.only(
                                    left: 12.0,
                                    right: 12.0,
                                    top: 16.0,
                                    bottom: 16.0),
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: [
                                    Text(
                                      "${participentList.length} participants",
                                      style: CustomTextStyles.medium(
                                          fontSize: 16.0,
                                          fontColor: const Color(0xff667B88)),
                                    ),
                                    createdBy.value == myUserId
                                        ? InkWell(
                                            onTap: () {
                                              // Get.to(
                                              //         () => ContactListView(
                                              //               forMakeGroup: true,
                                              //               forAddMember: true,
                                              //             ),
                                              //         arguments: [
                                              //       participentList,
                                              //       groupId.value
                                              //     ])!
                                              Navigator.push(
                                                context,
                                                MaterialPageRoute(
                                                    builder: (context) =>
                                                        ContactListView(
                                                            forMakeGroup: true,
                                                            forAddMember: true,
                                                            participentList:
                                                                participentList,
                                                            groupId:
                                                                groupId.value)),
                                              ).then((value) {
                                                if (value != null) {
                                                  getGroupDetail(groupId.value,
                                                      (data) {
                                                    participentList.value =
                                                        data['users'];
                                                    participentList.refresh();
                                                  });
                                                }
                                              });
                                            },
                                            child: Text(
                                              "+ Add Member",
                                              style: CustomTextStyles.medium(
                                                  fontSize: 16.0,
                                                  fontColor:
                                                      const Color(0xff0F659E)),
                                            ),
                                          )
                                        : SizedBox.shrink()
                                  ],
                                ),
                              ),
                              Obx(
                                () => ListView.separated(
                                    shrinkWrap: true,
                                    physics: NeverScrollableScrollPhysics(),
                                    padding: const EdgeInsets.only(
                                      left: 12.0,
                                      right: 12.0,
                                      bottom: 14.0,
                                    ),
                                    itemBuilder: (context, index) {
                                      return Row(
                                        children: [
                                          Common.profileNetworkImage(
                                            borderRadius: 56.0,
                                            height: 56.0,
                                            width: 56.0,
                                            placeHolder: ImageResource
                                                .profilePlaceHolder,
                                            imageUrl: participentList[index]
                                                    ['profile_picture'] ??
                                                "",
                                          ),
                                          const SizedBox(width: 10.0),
                                          Expanded(
                                            child: Text(
                                              participentList[index]
                                                          ['userId'] ==
                                                      myUserId
                                                  ? participentList[index]
                                                          ['name'] +
                                                      " (You)"
                                                  : participentList[index]
                                                      ['name'],
                                              style: CustomTextStyles.medium(
                                                  fontSize: 14.0,
                                                  fontColor: Colors.black),
                                              overflow: TextOverflow.ellipsis,
                                            ),
                                          ),
                                          participentList[index]['userId'] ==
                                                  createdBy.value
                                              ? Container(
                                                  decoration: BoxDecoration(
                                                    borderRadius:
                                                        BorderRadius.circular(
                                                            4),
                                                    color: themeColor,
                                                  ),
                                                  padding: const EdgeInsets
                                                          .symmetric(
                                                      vertical: 6,
                                                      horizontal: 6),
                                                  child: Text(
                                                    "Admin",
                                                    style:
                                                        CustomTextStyles.normal(
                                                      fontSize: 10.0,
                                                      fontColor:
                                                          Color(0xff0F659E),
                                                    ),
                                                  ),
                                                )
                                              : createdBy.value == myUserId
                                                  ? InkWell(
                                                      onTap: () {
                                                        var users = [];
                                                        var selectedIds = [];
                                                        participentList
                                                            .forEach((element) {
                                                          if (element[
                                                                  'userId'] !=
                                                              participentList[
                                                                      index]
                                                                  ['userId']) {
                                                            users.add({
                                                              "groups": element[
                                                                  'groups'],
                                                              "mobile_email":
                                                                  element[
                                                                      'mobile_email'],
                                                              "name": element[
                                                                  'name'],
                                                              "userId": element[
                                                                  'userId'],
                                                              "profilePicture":
                                                                  element[
                                                                      'profilePicture']
                                                            });
                                                            selectedIds.add(
                                                                element[
                                                                    'userId']);
                                                          }
                                                        });
                                                        var parameters = {
                                                          "secretKey": secKey,
                                                          "groupId":
                                                              groupId.value,
                                                          "users": users,
                                                          "members":
                                                              selectedIds,
                                                          "viewBy": selectedIds
                                                        };
                                                        removeGroupMember(
                                                            parameters, () {
                                                          getGroupDetail(
                                                              groupId.value,
                                                              (data) {
                                                            createdBy.value =
                                                                data[
                                                                    'createdBy'];
                                                            groupName.value =
                                                                data['name'];
                                                            groupNameController
                                                                    .text =
                                                                groupName.value;
                                                            participentList
                                                                    .value =
                                                                data['users'];
                                                            groupImage
                                                                .value = data[
                                                                    'groupImage'] ??
                                                                "";
                                                            participentList
                                                                .refresh();
                                                          });
                                                        });
                                                      },
                                                      child: Container(
                                                        decoration:
                                                            BoxDecoration(
                                                          borderRadius:
                                                              BorderRadius
                                                                  .circular(4),
                                                          color: const Color(
                                                              0xffFFE2E5),
                                                        ),
                                                        padding:
                                                            const EdgeInsets
                                                                    .symmetric(
                                                                vertical: 6,
                                                                horizontal: 6),
                                                        child: Text(
                                                          "Remove",
                                                          style:
                                                              CustomTextStyles
                                                                  .normal(
                                                            fontSize: 10.0,
                                                            fontColor:
                                                                const Color(
                                                                    0xffF64E89),
                                                          ),
                                                        ),
                                                      ),
                                                    )
                                                  : SizedBox.shrink()
                                        ],
                                      );
                                    },
                                    separatorBuilder: (context, index) {
                                      return const SizedBox(height: 16);
                                    },
                                    itemCount: participentList.length),
                              ),
                            ],
                          ),
                        ),
                        const SizedBox(height: 10),
                        InkWell(
                          onTap: () {
                            exitGroup(
                                userId: myUserId,
                                groupId: groupId.value,
                                context: context);
                          },
                          child: Container(
                            color: Colors.white,
                            padding: const EdgeInsets.all(20),
                            width: Get.width,
                            child: Row(
                              children: [
                                const Icon(
                                  Icons.login,
                                  color: Colors.red,
                                ),
                                const SizedBox(
                                  width: 8,
                                ),
                                Text(
                                  "Exit Group",
                                  style: CustomTextStyles.medium(
                                      fontSize: 16.0, fontColor: Colors.red),
                                )
                              ],
                            ),
                          ),
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        createdBy.value == myUserId
                            ? InkWell(
                                onTap: () {
                                  deleteGroup(
                                      userId: myUserId, groupId: groupId.value);
                                },
                                child: Container(
                                  color: Colors.white,
                                  padding: const EdgeInsets.all(20),
                                  width: Get.width,
                                  child: Row(
                                    children: [
                                      const Icon(
                                        Icons.delete,
                                        color: Colors.red,
                                      ),
                                      const SizedBox(
                                        width: 8,
                                      ),
                                      Text(
                                        "Delete Group",
                                        style: CustomTextStyles.medium(
                                            fontSize: 16.0,
                                            fontColor: Colors.red),
                                      )
                                    ],
                                  ),
                                ),
                              )
                            : const SizedBox.shrink()
                      ],
                    ),
                  ),
          ),
        ),
      ),
    );
  }

  var searchCommonBorderStyle = const OutlineInputBorder(
    borderSide: BorderSide(
      color: Color(0xffE6E6E6),
    ),
  );
  Widget appBar() {
    return Row(
      children: [
        IconButton(
            onPressed: () {
              Navigator.pop(context);
            },
            icon: const Icon(
              Icons.clear,
              size: 30.0,
            )),
        const SizedBox(
          width: 8,
        ),
        Expanded(
          child: Text(
            "Group Info",
            style: CustomTextStyles.medium(
                fontSize: 18.0, fontColor: Colors.black),
          ),
        ),
        createdBy.value == myUserId
            ? Padding(
                padding: EdgeInsets.only(right: 10),
                child: InkWell(
                  onTap: () {
                    if (isEditable.value) {
                      var param = {
                        "groupId": groupId.value,
                        "name": groupNameController.text,
                        "profilePicture": imageBytes
                      };
                      updateGroupInfo(param, context);
                      isEditable.value = false;
                    } else {
                      isEditable.value = true;
                    }
                  },
                  child: Obx(
                    () => Text(isEditable.value ? "Save" : "Edit",
                        style: CustomTextStyles.medium(
                          fontSize: 20.0,
                          fontColor: Colors.black,
                        )),
                  ),
                ),
              )
            : SizedBox.shrink()
      ],
    );
  }
}
